/*
    mandelbmp - render the Mandelbrot set or Julia sets as BMP files
    Copyright (C) 2022 cas <cas@sdf.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef bmp_h
#define bmp_h
#include <stdint.h>
#include <stdio.h>

#define BI_RGB       0
#define BI_RLE8      1
#define BI_RLE4      2
#define BI_BITFIELDS 3

#pragma pack(1)

/*
 * Structs for bitmap files. Modeled after german wikipedia article on
 * the BMP format.
 * 
 * https://de.wikipedia.org/wiki/Windows_Bitmap
 * 
 * */

typedef struct{
    uint16_t bfType;
    uint32_t bfSize;
    uint32_t bfReserved;
    uint32_t bfOffBits;
    
} BMPFileHeader;

typedef struct{
    uint32_t biSize;
    int32_t biWidth;
    int32_t biHeight;
    uint16_t biPlanes;
    uint16_t biBitCount;
    uint32_t biCompression;
    uint32_t biSizeImage;
    int32_t biXPelsPerMeter;
    int32_t biYPelsPerMeter;
    uint32_t biClrUsed;
    uint32_t biClrImportant;
} BMPInfoHeader;

// this will be the full bitmap image then
typedef struct{
  BMPFileHeader* file_header;
  BMPInfoHeader* info_header;
  
  uint8_t* image_data;
} BMP;

#pragma pack(0)

void print_BMPFileHeader(BMP* bmp);
void print_BMPInfoHeader(BMP* bmp);
BMP* load_BMP(char* fname);
BMP* create_BMP(unsigned int w, unsigned int h);
int  write_BMP(char* fname, BMP* bmp);
void free_BMP(BMP* bmp);

void set_rgb(unsigned int x,
             unsigned int y,
             uint8_t r,
             uint8_t g,
             uint8_t b,
             BMP* bmp);

#endif
